@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 from libnmap.diff import NmapDiff
+from libnmap.objects.os import CPE
 
 
 class NmapService(object):
@@ -11,7 +12,7 @@ class NmapService(object):
         available or not. Like banner or extra datas from NSE (nmap scripts).
     """
     def __init__(self, portid, protocol='tcp', state=None,
-                 service=None, service_extras=None):
+                 service=None, owner=None, service_extras=None):
         """
             Constructor
 
@@ -37,6 +38,16 @@ class NmapService(object):
         self._state = state if state is not None else {}
         self._service = service if service is not None else {}
 
+        self._cpelist = []
+        if 'cpelist' in self._service:
+            for _cpe in self._service['cpelist']:
+                _cpeobj = CPE(_cpe)
+                self._cpelist.append(_cpeobj)
+
+        self._owner = ''
+        if owner is not None and 'name' in owner:
+            self._owner = owner['name']
+
         self._reason = ''
         self._reason_ip = ''
         self._reason_ttl = ''
@@ -170,7 +181,7 @@ class NmapService(object):
 
             :return: dict or None
         """
-        return self._service['name'] if 'name' in self._service else None
+        return self._service['name'] if 'name' in self._service else ''
 
     def open(self):
         """
@@ -180,6 +191,13 @@ class NmapService(object):
         """
         return 'state' in self._state and self._state['state'] == 'open'
 
+    @property
+    def owner(self):
+        """
+            Accessor for service owner if available
+        """
+        return self._owner
+
     @property
     def banner(self):
         """
@@ -188,7 +206,7 @@ class NmapService(object):
 
             :return: string
         """
-        notrelevant = ['name', 'method', 'conf']
+        notrelevant = ['name', 'method', 'conf', 'cpelist']
         b = ''
         if 'method' in self._service and self._service['method'] == "probed":
             b = " ".join([k + ": " + self._service[k]
@@ -196,6 +214,13 @@ class NmapService(object):
                               if k not in notrelevant])
         return b
 
+    @property
+    def cpelist(self):
+        """
+            Accessor for list of CPE for this particular service
+        """
+        return self._cpelist
+
     @property
     def scripts_results(self):
         """
