@@ -1,13 +1,19 @@
 #!/usr/bin/env python
 import json
 from libnmap.objects import NmapHost, NmapService, NmapReport
+from libnmap.objects.os import NmapOSFingerprint, NmapOSMatch, NmapOSClass
+from libnmap.objects.os import CPE, OSFPPortUsed
 from libnmap.parser import NmapParser
 
 
 class ReportEncoder(json.JSONEncoder):
     def default(self, obj):
         otype = {'NmapHost': NmapHost,
-                 'NmapOSFingerprint': NmapHost.NmapOSFingerprint,
+                 'NmapOSFingerprint': NmapOSFingerprint,
+                 'NmapOSMatch': NmapOSMatch,
+                 'NmapOSClass': NmapOSClass,
+                 'CPE': CPE,
+                 'OSFPPortUsed': OSFPPortUsed,
                  'NmapService': NmapService,
                  'NmapReport': NmapReport}
         if isinstance(obj, tuple(otype.values())):
